@@ -50,12 +50,11 @@
    (G_TYPE_INSTANCE_GET_PRIVATE ((o), FS_TYPE_PLUGIN, FsPluginPrivate))
 
 static gboolean fs_plugin_load (GTypeModule *module);
-static void fs_plugin_unload (GTypeModule *module);
 
 
 static gchar **search_paths = NULL;
 
-GList *plugins = NULL;
+static GList *plugins = NULL;
 
 static GObjectClass *parent_class = NULL;
 
@@ -71,7 +70,7 @@ static void fs_plugin_dispose (GObject * object);
 static void fs_plugin_finalize (GObject * object);
 
 static void
-fs_plugin_search_path_init ()
+fs_plugin_search_path_init (void)
 {
   const gchar *env;
 
@@ -106,10 +105,9 @@ fs_plugin_class_init (FsPluginClass * klass)
   gobject_class->dispose = fs_plugin_dispose;
   gobject_class->finalize = fs_plugin_finalize;
 
-  parent_class = g_type_class_peek_parent(klass);
+  parent_class = g_type_class_peek_parent (klass);
 
   module_class->load = fs_plugin_load;
-  module_class->unload = fs_plugin_unload;
 
   g_type_class_add_private (klass, sizeof (FsPluginPrivate));
 
@@ -165,7 +163,7 @@ static gboolean fs_plugin_load (GTypeModule *module)
   for (search_path = search_paths; *search_path; search_path++) {
     GST_DEBUG("looking for plugins in %s", *search_path);
 
-    path = g_module_build_path(*search_path, plugin->name);
+    path = g_module_build_path (*search_path, plugin->name);
 
     plugin->priv->handle = g_module_open (path, G_MODULE_BIND_LOCAL);
     GST_INFO ("opening module %s: %s\n", path,
@@ -208,24 +206,6 @@ static gboolean fs_plugin_load (GTypeModule *module)
 
 }
 
-static void
-fs_plugin_unload (GTypeModule *module)
-{
-  FsPlugin *plugin = NULL;
-
-  g_return_if_fail (module != NULL);
-
-  plugin = FS_PLUGIN (module);
-
-  GST_INFO("Unloading plugin %s", plugin->name);
-
-  if (plugin->unload != NULL)
-    plugin->unload (plugin);
-
-  if (plugin->priv->handle != NULL)
-    g_module_close (plugin->priv->handle);
-}
-
 static FsPlugin *
 fs_plugin_get_by_name (const gchar * name, const gchar * type_suffix)
 {
@@ -240,7 +220,7 @@ fs_plugin_get_by_name (const gchar * name, const gchar * type_suffix)
 
   for (plugin_item = plugins;
        plugin_item;
-       plugin_item = g_list_next(plugin_item))  {
+       plugin_item = g_list_next (plugin_item))  {
     plugin = plugin_item->data;
     if (plugin->name == NULL || plugin->name[0] == 0)
       continue;
@@ -280,18 +260,10 @@ fs_plugin_create_valist (const gchar *name, const gchar *type_suffix,
   GObject *object;
   FsPlugin *plugin;
 
-  fs_base_conference_init_debug ();
+  g_return_val_if_fail (name, NULL);
+  g_return_val_if_fail (type_suffix, NULL);
 
-  if (name == NULL) {
-    g_set_error (error, FS_ERROR, FS_ERROR_INVALID_ARGUMENTS,
-      "You need to pass a name");
-    return NULL;
-  }
-  if (type_suffix == NULL) {
-    g_set_error (error, FS_ERROR, FS_ERROR_INVALID_ARGUMENTS,
-      "You need to pass a type suffix");
-    return NULL;
-  }
+  fs_base_conference_init_debug ();
 
   plugin = fs_plugin_get_by_name (name, type_suffix);
 
@@ -304,19 +276,19 @@ fs_plugin_create_valist (const gchar *name, const gchar *type_suffix,
     }
     plugin->name = g_strdup_printf ("%s-%s",name,type_suffix);
     g_type_module_set_name (G_TYPE_MODULE (plugin), plugin->name);
-    plugins = g_list_append(plugins, plugin);
-  }
+    plugins = g_list_append (plugins, plugin);
 
-  if (!g_type_module_use (G_TYPE_MODULE (plugin))) {
-    g_set_error (error, FS_ERROR, FS_ERROR_CONSTRUCTION,
-      "Could not load the %s-%s transmitter plugin", name, type_suffix);
-    return NULL;
+    /* We do the use once and then we keep it loaded forever because
+     * the gstreamer libraries can't be unloaded
+     */
+    if (!g_type_module_use (G_TYPE_MODULE (plugin))) {
+      g_set_error (error, FS_ERROR, FS_ERROR_CONSTRUCTION,
+          "Could not load the %s-%s transmitter plugin", name, type_suffix);
+      return NULL;
+    }
   }
-
   object = g_object_new_valist (plugin->type, first_property_name, var_args);
 
-  g_type_module_unuse(G_TYPE_MODULE(plugin));
-
   return object;
 }
 
@@ -328,7 +300,7 @@ fs_plugin_create_valist (const gchar *name, const gchar *type_suffix,
  * @error: location of a #GError, or NULL if no error occured
  * @first_property_name: The name of the first property to be set on the
  *   object
- * @...: The NULL-terminated list of properties to set on the transmitter 
+ * @...: The NULL-terminated list of properties to set on the transmitter
  *
  * Loads the appropriate plugin if necessary and creates a GObject of
  * the requested type
@@ -350,3 +322,93 @@ fs_plugin_create (const gchar *name, const gchar *type_suffix,
 
   return obj;
 }
+
+/**
+ * fs_plugin_list_available:
+ * @type_suffix: Get list of plugins with this type suffix
+ *
+ * Gets the list of all available plugins of a certain type
+ *
+ * Returns: a newly allocated NULL terminated array of strings or %NULL if no
+ * strings were found. It should be freed with g_strfreev().
+ */
+
+gchar **
+fs_plugin_list_available (const gchar *type_suffix)
+{
+  GPtrArray *list = g_ptr_array_new ();
+  gchar **retval = NULL;
+  gchar **search_path = NULL;
+  GRegex *matcher;
+  GError *error = NULL;
+  gchar *tmp1, *tmp2, *tmp3;
+
+  fs_plugin_search_path_init ();
+
+  tmp1 = g_strdup_printf ("(.+)-%s", type_suffix);
+  tmp2 = g_module_build_path ("", tmp1);
+  tmp3 = g_strconcat ("^", tmp2, NULL);
+  matcher = g_regex_new (tmp3, 0, 0, NULL);
+  g_free (tmp1);
+  g_free (tmp2);
+  g_free (tmp3);
+
+
+  for (search_path = search_paths; *search_path; search_path++)
+  {
+    GDir *dir = NULL;
+    const gchar *entry;
+
+    dir = g_dir_open (*search_path, 0, &error);
+    if (!dir)
+    {
+      GST_WARNING ("Could not open path %s to look for plugins: %s",
+          search_path, error ? error->message : "Unknown error");
+      g_clear_error (&error);
+      continue;
+    }
+
+    while ((entry = g_dir_read_name (dir)))
+    {
+      gchar **matches = NULL;
+
+      matches = g_regex_split (matcher, entry, 0);
+
+      if (matches && g_strv_length (matches) == 3)
+      {
+        gint i;
+        gboolean found = FALSE;
+
+        for (i = 0; i < list->len; i++)
+        {
+          if (!strcmp (matches[1], g_ptr_array_index (list, i)))
+          {
+            found = TRUE;
+            break;
+          }
+        }
+        if (!found)
+          g_ptr_array_add (list, g_strdup (matches[1]));
+      }
+
+      g_strfreev (matches);
+    }
+
+    g_dir_close (dir);
+  }
+
+  g_regex_unref (matcher);
+
+  if (list->len)
+  {
+    g_ptr_array_add (list, NULL);
+    retval = (gchar**) list->pdata;
+    g_ptr_array_free (list, FALSE);
+  }
+  else
+  {
+    g_ptr_array_free (list, TRUE);
+  }
+
+  return retval;
+}
