@@ -73,6 +73,8 @@ struct _FsRtpStreamPrivate
 
   stream_new_remote_codecs_cb new_remote_codecs_cb;
   stream_known_source_packet_receive_cb known_source_packet_received_cb;
+  stream_sending_changed_locked_cb sending_changed_locked_cb;
+  stream_ssrc_added_cb ssrc_added_cb;
   gpointer user_data_for_cb;
 
   GMutex *mutex;
@@ -109,6 +111,8 @@ static gboolean fs_rtp_stream_set_remote_codecs (FsStream *stream,
                                                  GList *remote_codecs,
                                                  GError **error);
 
+static void fs_rtp_stream_add_id (FsStream *stream, guint id);
+
 static void _local_candidates_prepared (
     FsStreamTransmitter *stream_transmitter,
     gpointer user_data);
@@ -161,6 +165,7 @@ fs_rtp_stream_class_init (FsRtpStreamClass *klass)
   stream_class->set_remote_candidates = fs_rtp_stream_set_remote_candidates;
   stream_class->set_remote_codecs = fs_rtp_stream_set_remote_codecs;
   stream_class->force_remote_candidates = fs_rtp_stream_force_remote_candidates;
+  stream_class->add_id = fs_rtp_stream_add_id;
 
 
   g_type_class_add_private (klass, sizeof (FsRtpStreamPrivate));
@@ -261,6 +266,12 @@ fs_rtp_stream_dispose (GObject *object)
   g_mutex_unlock (self->priv->mutex);
 
   FS_RTP_SESSION_LOCK (session);
+
+  if (self->priv->sending_changed_locked_cb &&
+      self->priv->direction & FS_DIRECTION_SEND)
+    self->priv->sending_changed_locked_cb (self, FALSE,
+        self->priv->user_data_for_cb);
+
   participant = self->participant;
   self->participant = NULL;
 
@@ -419,6 +430,13 @@ fs_rtp_stream_set_property (GObject *object,
         }
 
         FS_RTP_SESSION_LOCK (session);
+        if (self->priv->sending_changed_locked_cb &&
+            (self->priv->direction & FS_DIRECTION_SEND) !=
+            (g_value_get_flags (value) & FS_DIRECTION_SEND))
+          self->priv->sending_changed_locked_cb (self,
+              g_value_get_flags (value) & FS_DIRECTION_SEND,
+              self->priv->user_data_for_cb);
+
         dir = self->priv->direction = g_value_get_flags (value);
         FS_RTP_SESSION_UNLOCK (session);
         st = fs_rtp_stream_get_stream_transmitter (self, NULL);
@@ -605,12 +623,6 @@ fs_rtp_stream_set_remote_codecs (FsStream *stream,
           codec->encoding_name);
       goto error;
     }
-    if (codec->clock_rate == 0)
-    {
-      g_set_error (error, FS_ERROR, FS_ERROR_INVALID_ARGUMENTS,
-          "The codec %s must have a non-0 clock rate", codec->encoding_name);
-      goto error;
-    }
     if (codec->media_type != media_type)
     {
       g_set_error (error, FS_ERROR, FS_ERROR_INVALID_ARGUMENTS,
@@ -623,11 +635,19 @@ fs_rtp_stream_set_remote_codecs (FsStream *stream,
   if (self->priv->new_remote_codecs_cb (self, remote_codecs, error,
           self->priv->user_data_for_cb))
   {
+    gboolean is_new = TRUE;
+
     FS_RTP_SESSION_LOCK (session);
     if (self->remote_codecs)
+    {
+      is_new = !fs_codec_list_are_equal (self->remote_codecs, remote_codecs);
       fs_codec_list_destroy (self->remote_codecs);
+    }
     self->remote_codecs = fs_codec_list_copy (remote_codecs);
     FS_RTP_SESSION_UNLOCK (session);
+
+    if (is_new)
+      g_object_notify (G_OBJECT (stream), "remote-codecs");
   } else {
     goto error;
   }
@@ -653,6 +673,8 @@ fs_rtp_stream_set_remote_codecs (FsStream *stream,
  * the session lock across calls.
  * @known_source_packet_received: Callback called when a packet from a
  * known source is receive.
+ * @sending_changed_locked_cb: Callback called when the sending status of
+ *  this stream changes
  * @user_data: User data for the callbacks.
  * This function create a new stream
  *
@@ -666,6 +688,8 @@ fs_rtp_stream_new (FsRtpSession *session,
     FsStreamTransmitter *stream_transmitter,
     stream_new_remote_codecs_cb new_remote_codecs_cb,
     stream_known_source_packet_receive_cb known_source_packet_received_cb,
+    stream_sending_changed_locked_cb sending_changed_locked_cb,
+    stream_ssrc_added_cb ssrc_added_cb,
     gpointer user_data_for_cb,
     GError **error)
 {
@@ -686,8 +710,16 @@ fs_rtp_stream_new (FsRtpSession *session,
 
   self->priv->new_remote_codecs_cb = new_remote_codecs_cb;
   self->priv->known_source_packet_received_cb = known_source_packet_received_cb;
+  self->priv->sending_changed_locked_cb = sending_changed_locked_cb;
+  self->priv->ssrc_added_cb = ssrc_added_cb;
   self->priv->user_data_for_cb = user_data_for_cb;
 
+  FS_RTP_SESSION_LOCK (session);
+  if (sending_changed_locked_cb && (direction & FS_DIRECTION_SEND))
+    sending_changed_locked_cb (self, direction & FS_DIRECTION_SEND,
+        user_data_for_cb);
+  FS_RTP_SESSION_UNLOCK (session);
+
   if (self->priv->construction_error) {
     g_propagate_error (error, self->priv->construction_error);
     g_object_unref (self);
@@ -853,6 +885,28 @@ _substream_error (FsRtpSubStream *substream,
   fs_stream_emit_error (stream, errorno, error_msg, debug_msg);
 }
 
+
+static void
+_substream_unlinked (FsRtpSubStream *substream, gpointer user_data)
+{
+  FsRtpStream *stream = FS_RTP_STREAM (user_data);
+  FsRtpSession *session =  fs_rtp_stream_get_session (stream, NULL);
+
+  if (!session)
+    return;
+
+  FS_RTP_SESSION_LOCK (session);
+  stream->substreams = g_list_remove (stream->substreams,
+      substream);
+  FS_RTP_SESSION_UNLOCK (session);
+
+  fs_rtp_sub_stream_stop (substream);
+
+  g_object_unref (substream);
+  g_object_unref (session);
+}
+
+
 /**
  * fs_rtp_stream_add_substream_unlock:
  * @stream: a #FsRtpStream
@@ -883,6 +937,8 @@ fs_rtp_stream_add_substream_unlock (FsRtpStream *stream,
       "receiving", ((stream->priv->direction & FS_DIRECTION_RECV) != 0),
       NULL);
 
+  g_signal_connect (substream, "unlinked",
+      G_CALLBACK (_substream_unlinked), stream);
   g_signal_connect (substream, "src-pad-added",
                     G_CALLBACK (_substream_src_pad_added), stream);
   g_signal_connect (substream, "codec-changed",
@@ -890,6 +946,8 @@ fs_rtp_stream_add_substream_unlock (FsRtpStream *stream,
   g_signal_connect (substream, "error",
                     G_CALLBACK (_substream_error), stream);
 
+  fs_rtp_sub_stream_verify_codec (substream);
+
   /* Only announce a pad if it has a codec attached to it */
   if (substream->codec)
     ret = fs_rtp_sub_stream_add_output_ghostpad_unlock (substream, error);
@@ -1016,3 +1074,18 @@ fs_rtp_stream_set_negotiated_codecs_unlock (FsRtpStream *stream,
 
   g_object_unref (session);
 }
+
+static void
+fs_rtp_stream_add_id (FsStream *stream, guint id)
+{
+  FsRtpStream *self = FS_RTP_STREAM (stream);
+  FsRtpSession *session = fs_rtp_stream_get_session (self, NULL);
+
+  if (!session)
+    return;
+
+  if (self->priv->ssrc_added_cb)
+    self->priv->ssrc_added_cb (self, id, self->priv->user_data_for_cb);
+
+  g_object_unref (session);
+}
