@@ -17,10 +17,11 @@ filter=pyfarsight-filter.defs
 
 cat ${filter} > ${output}
 
-H2DEF=
+
+
+H2DEF="$(pkg-config --variable=codegendir pygobject-2.0)/h2def.py"
+[ -z "${H2DEF}" ] && H2DEF="$(pkg-config --variable=codegendir pygtk-2.0)/h2def.py"
 [ -z "${H2DEF}" -a -f /usr/share/pygtk/2.0/codegen/h2def.py ] && H2DEF=/usr/share/pygtk/2.0/codegen/h2def.py
-[ -z "${H2DEF}" -a -f /usr/lib/python2.5/site-packages/gtk-2.0/codegen/h2def.py ] && H2DEF=/usr/lib/python2.5/site-packages/gtk-2.0/codegen/h2def.py
-[ -z "${H2DEF}" -a -f /usr/lib/python2.4/site-packages/gtk-2.0/codegen/h2def.py ] && H2DEF=/usr/lib/python2.4/site-packages/gtk-2.0/codegen/h2def.py
 
 for h in $HEADERS; do
     python ${H2DEF} --defsfilter=${filter} ${srcdir}/$h >> $output
