@@ -26,15 +26,10 @@
 #include <gst/farsight/fs-transmitter.h>
 #include <gst/farsight/fs-conference-iface.h>
 
-#ifdef HAVE_GETIFADDRS
- #include <sys/socket.h>
- #include <ifaddrs.h>
- #include <net/if.h>
- #include <arpa/inet.h>
-#endif
-
 #include "check-threadsafe.h"
 #include "generic.h"
+#include "fake-filter.h"
+#include "testutils.h"
 
 gint buffer_count[2] = {0, 0};
 GMainLoop *loop = NULL;
@@ -42,6 +37,12 @@ gint candidates[2] = {0, 0};
 GstElement *pipeline = NULL;
 gboolean src_setup[2] = {FALSE, FALSE};
 
+enum {
+  FLAG_NOT_SENDING = 1<<0,
+  FLAG_RECVONLY_FILTER = 1<<1
+};
+
+
 GST_START_TEST (test_multicasttransmitter_new)
 {
   test_transmitter_creation ("multicast");
@@ -107,8 +108,19 @@ _start_pipeline (gpointer user_data)
   return FALSE;
 }
 
+
+static GstElement *
+_get_recvonly_filter (FsTransmitter *trans, guint component, gpointer user_data)
+{
+  if (component == 1)
+    return NULL;
+
+  return gst_element_factory_make ("fsfakefilter", NULL);
+}
+
 static void
-run_multicast_transmitter_test (gint n_parameters, GParameter *params)
+run_multicast_transmitter_test (gint n_parameters, GParameter *params,
+                                gint flags)
 {
   GError *error = NULL;
   FsTransmitter *trans;
@@ -117,6 +129,15 @@ run_multicast_transmitter_test (gint n_parameters, GParameter *params)
   GList *candidates = NULL;
   GstBus *bus = NULL;
 
+  buffer_count[0] = 0;
+  buffer_count[1] = 0;
+
+  if (flags & FLAG_NOT_SENDING && flags & FLAG_RECVONLY_FILTER)
+    buffer_count[0] = 20;
+
+  if (flags & FLAG_RECVONLY_FILTER)
+    fail_unless (fs_fake_filter_register ());
+
   loop = g_main_loop_new (NULL, FALSE);
   trans = fs_transmitter_new ("multicast", 2, &error);
 
@@ -127,6 +148,11 @@ run_multicast_transmitter_test (gint n_parameters, GParameter *params)
 
   ts_fail_if (trans == NULL, "No transmitter create, yet error is still NULL");
 
+  if (flags & FLAG_RECVONLY_FILTER)
+    ts_fail_unless (g_signal_connect (trans, "get-recvonly-filter",
+            G_CALLBACK (_get_recvonly_filter), NULL));
+
+
   pipeline = setup_pipeline (trans, G_CALLBACK (_handoff_handler));
 
   st = fs_transmitter_new_stream_transmitter (trans, NULL, n_parameters, params,
@@ -139,6 +165,8 @@ run_multicast_transmitter_test (gint n_parameters, GParameter *params)
 
   ts_fail_if (st == NULL, "No stream transmitter created, yet error is NULL");
 
+  g_object_set (st, "sending", !(flags & FLAG_NOT_SENDING), NULL);
+
   bus = gst_element_get_bus (pipeline);
   gst_bus_add_watch (bus, bus_error_callback, NULL);
   gst_object_unref (bus);
@@ -174,7 +202,7 @@ run_multicast_transmitter_test (gint n_parameters, GParameter *params)
 
   fs_candidate_list_destroy (candidates);
 
-  g_main_run (loop);
+  g_main_loop_run (loop);
 
   g_object_unref (st);
 
@@ -189,64 +217,17 @@ run_multicast_transmitter_test (gint n_parameters, GParameter *params)
 
 GST_START_TEST (test_multicasttransmitter_run)
 {
-  run_multicast_transmitter_test (0, NULL);
+  run_multicast_transmitter_test (0, NULL, 0);
 }
 GST_END_TEST;
 
-static gchar *
-_find_multicast_capable_address (void)
-{
-#ifdef HAVE_GETIFADDRS
-  gchar *retval = NULL;
-  struct ifaddrs *ifa, *results;
-
-  if (getifaddrs (&results) < 0)
-    return NULL;
-
-  for (ifa = results; ifa; ifa = ifa->ifa_next) {
-    /* no ip address from interface that is down */
-    if ((ifa->ifa_flags & IFF_UP) == 0)
-      continue;
-
-    if ((ifa->ifa_flags & IFF_MULTICAST) == 0)
-      continue;
-
-    if (ifa->ifa_addr == NULL || ifa->ifa_addr->sa_family != AF_INET)
-      continue;
-
-    if (retval)
-    {
-      g_free (retval);
-      retval = NULL;
-      g_debug ("Disabling test, more than one multicast capable interface");
-      break;
-    }
-
-    retval = g_strdup (
-        inet_ntoa (((struct sockaddr_in *) ifa->ifa_addr)->sin_addr));
-    g_debug ("Sending from %s on interface %s", retval, ifa->ifa_name);
-  }
-
-  freeifaddrs (results);
-
-  if (retval == NULL)
-    g_message ("Skipping multicast transmitter tests, "
-        "no multicast capable interface found");
-  return retval;
-
-#else
-  g_message ("This system does not have getifaddrs,"
-      " this test will be disabled");
-  return NULL;
-#endif
-}
 
 GST_START_TEST (test_multicasttransmitter_run_local_candidates)
 {
   GParameter params[1];
   GList *list = NULL;
   FsCandidate *candidate;
-  gchar *address = _find_multicast_capable_address ();
+  gchar *address = find_multicast_capable_address ();
 
   if (address == NULL)
     return;
@@ -267,7 +248,7 @@ GST_START_TEST (test_multicasttransmitter_run_local_candidates)
   g_value_init (&params[0].value, FS_TYPE_CANDIDATE_LIST);
   g_value_set_boxed (&params[0].value, list);
 
-  run_multicast_transmitter_test (1, params);
+  run_multicast_transmitter_test (1, params, 0);
 
   g_value_reset (&params[0].value);
 
@@ -276,6 +257,22 @@ GST_START_TEST (test_multicasttransmitter_run_local_candidates)
 }
 GST_END_TEST;
 
+GST_START_TEST (test_multicasttransmitter_with_filter)
+{
+  run_multicast_transmitter_test (0, NULL,
+      FLAG_RECVONLY_FILTER);
+}
+GST_END_TEST;
+
+GST_START_TEST (test_multicasttransmitter_sending_half)
+{
+  run_multicast_transmitter_test (0, NULL,
+      FLAG_NOT_SENDING | FLAG_RECVONLY_FILTER);
+}
+GST_END_TEST;
+
+
+
 static Suite *
 multicasttransmitter_suite (void)
 {
@@ -284,8 +281,7 @@ multicasttransmitter_suite (void)
   GLogLevelFlags fatal_mask;
   gchar *tmp_addr;
 
-
-  tmp_addr = _find_multicast_capable_address ();
+  tmp_addr = find_multicast_capable_address ();
 
   if (!tmp_addr)
     return s;
@@ -305,6 +301,14 @@ multicasttransmitter_suite (void)
   tcase_add_test (tc_chain, test_multicasttransmitter_run_local_candidates);
   suite_add_tcase (s, tc_chain);
 
+  tc_chain = tcase_create ("multicast_transmitter-with-filter");
+  tcase_add_test (tc_chain, test_multicasttransmitter_with_filter);
+  suite_add_tcase (s, tc_chain);
+
+  tc_chain = tcase_create ("multicast_transmitter_sending_half");
+  tcase_add_test (tc_chain, test_multicasttransmitter_sending_half);
+  suite_add_tcase (s, tc_chain);
+
   return s;
 }
 
