@@ -30,12 +30,15 @@
 
 #include "check-threadsafe.h"
 #include "generic.h"
+#include "fake-filter.h"
 
 
 enum {
   FLAG_NO_SOURCE = 1 << 0,
   FLAG_IS_LOCAL = 1 << 1,
-  FLAG_FORCE_CANDIDATES = 1 << 2
+  FLAG_FORCE_CANDIDATES = 1 << 2,
+  FLAG_NOT_SENDING = 1 << 3,
+  FLAG_RECVONLY_FILTER = 1 << 4
 };
 
 
@@ -70,8 +73,16 @@ _new_local_candidate (FsStreamTransmitter *st, FsCandidate *candidate,
   ts_fail_if (candidate->foundation == NULL,
       "Candidate doenst have a foundation");
   ts_fail_if (candidate->component_id == 0, "Component id is 0");
-  ts_fail_if (candidate->base_ip == NULL, "Candidate doesnt have a base ip");
-  ts_fail_if (candidate->base_port == 0, "Candidate doesnt have a base port");
+  if (candidate->type == FS_CANDIDATE_TYPE_HOST)
+  {
+    ts_fail_if (candidate->base_ip != NULL, "Host candidate has a base ip");
+    ts_fail_if (candidate->base_port != 0, "Host candidate has a base port");
+  }
+  else
+  {
+    ts_fail_if (candidate->base_ip == NULL, "Candidate doesnt have a base ip");
+    ts_fail_if (candidate->base_port == 0, "Candidate doesnt have a base port");
+  }
   ts_fail_if (candidate->username == NULL, "Candidate doenst have a username");
   ts_fail_if (candidate->password == NULL, "Candidate doenst have a password");
 
@@ -186,13 +197,15 @@ _handoff_handler (GstElement *element, GstBuffer *buffer, GstPad *pad,
   buffer_count[stream][component_id-1]++;
 
 
-  g_debug ("Buffer %d stream: %u component: %d size: %u",
-      buffer_count[stream][component_id-1], stream,
-      component_id, GST_BUFFER_SIZE (buffer));
-
-  g_debug ("has %d %d %d %d",
-      buffer_count[0][0], buffer_count[0][1],
-      buffer_count[1][0], buffer_count[1][1]);
+  if (buffer_count[stream][component_id-1] % 10 == 0)
+  {
+    g_debug ("Buffer %d stream: %u component: %d size: %u",
+        buffer_count[stream][component_id-1], stream,
+        component_id, GST_BUFFER_SIZE (buffer));
+    g_debug ("Received %d %d %d %d",
+        buffer_count[0][0], buffer_count[0][1],
+        buffer_count[1][0], buffer_count[1][1]);
+  }
 
   ts_fail_if (buffer_count[stream][component_id-1] > 20,
     "Too many buffers %d > 20 for component",
@@ -343,6 +356,14 @@ fs_nice_test_participant_class_init (FsNiceTestParticipantClass *klass)
 {
 }
 
+static GstElement *
+_get_recvonly_filter (FsTransmitter *trans, guint component, gpointer user_data)
+{
+  if (component == 1)
+    return NULL;
+
+  return gst_element_factory_make ("fsfakefilter", NULL);
+}
 
 static void
 run_nice_transmitter_test (gint n_parameters, GParameter *params,
@@ -364,6 +385,17 @@ run_nice_transmitter_test (gint n_parameters, GParameter *params,
   is_address_local = (flags & FLAG_IS_LOCAL);
   force_candidates = (flags & FLAG_FORCE_CANDIDATES);
 
+  if (flags & FLAG_RECVONLY_FILTER)
+    fail_unless (fs_fake_filter_register ());
+
+  if (flags & FLAG_NOT_SENDING)
+  {
+    buffer_count[0][0] = 20;
+    received_known[0][0] = 20;
+    buffer_count[1][0] = 20;
+    received_known[1][0] = 20;
+  }
+
   loop = g_main_loop_new (NULL, FALSE);
 
   trans = fs_transmitter_new ("nice", 2, &error);
@@ -373,6 +405,10 @@ run_nice_transmitter_test (gint n_parameters, GParameter *params,
   }
   ts_fail_if (trans == NULL, "No transmitter create, yet error is still NULL");
 
+  if (flags & FLAG_RECVONLY_FILTER)
+    ts_fail_unless (g_signal_connect (trans, "get-recvonly-filter",
+            G_CALLBACK (_get_recvonly_filter), NULL));
+
   trans2 = fs_transmitter_new ("nice", 2, &error);
   if (error) {
     ts_fail ("Error creating transmitter: (%s:%d) %s",
@@ -380,6 +416,9 @@ run_nice_transmitter_test (gint n_parameters, GParameter *params,
   }
   ts_fail_if (trans2 == NULL, "No transmitter create, yet error is still NULL");
 
+ if (flags & FLAG_RECVONLY_FILTER)
+    ts_fail_unless (g_signal_connect (trans2, "get-recvonly-filter",
+            G_CALLBACK (_get_recvonly_filter), NULL));
 
   pipeline = setup_pipeline (trans, G_CALLBACK (_handoff_handler1));
   pipeline2 = setup_pipeline (trans2, G_CALLBACK (_handoff_handler2));
@@ -417,6 +456,9 @@ run_nice_transmitter_test (gint n_parameters, GParameter *params,
         g_quark_to_string (error->domain), error->code, error->message);
   ts_fail_if (st2 == NULL, "No stream transmitter created, yet error is NULL");
 
+  g_object_set (st, "sending", !(flags & FLAG_NOT_SENDING), NULL);
+  g_object_set (st2, "sending", !(flags & FLAG_NOT_SENDING), NULL);
+
   ts_fail_unless (g_signal_connect (st, "new-local-candidate",
       G_CALLBACK (_new_local_candidate), st2),
     "Could not connect new-local-candidate signal");
@@ -484,7 +526,7 @@ run_nice_transmitter_test (gint n_parameters, GParameter *params,
 
   g_idle_add (check_running, NULL);
 
-  g_main_run (loop);
+  g_main_loop_run (loop);
 
   gst_element_set_state (pipeline, GST_STATE_NULL);
   gst_element_get_state (pipeline, NULL, NULL, GST_CLOCK_TIME_NONE);
@@ -682,7 +724,11 @@ GST_START_TEST (test_nicetransmitter_invalid_arguments)
 
   /* no IP */
   va = g_value_array_new (1);
-  s = gst_structure_new ("aa", NULL);
+  s = gst_structure_new ("aa",
+      "port", G_TYPE_UINT, 7654,
+      "username", G_TYPE_STRING, "blah",
+      "password", G_TYPE_STRING, "blah2",
+      NULL);
   g_value_init (&val, GST_TYPE_STRUCTURE);
   g_value_take_boxed (&val, s);
   g_value_array_append (va, &val);
@@ -698,6 +744,61 @@ GST_START_TEST (test_nicetransmitter_invalid_arguments)
   va = g_value_array_new (1);
   s = gst_structure_new ("aa",
       "ip", G_TYPE_STRING, "127.0.0.1",
+      "username", G_TYPE_STRING, "blah",
+      "password", G_TYPE_STRING, "blah2",
+      NULL);
+  g_value_take_boxed (&val, s);
+  g_value_array_append (va, &val);
+  g_value_take_boxed (&params[0].value, va);
+  st = fs_transmitter_new_stream_transmitter (trans, p, 1, params, &error);
+  ts_fail_unless (st == NULL);
+  ts_fail_unless (error &&
+      error->domain == FS_ERROR &&
+      error->code == FS_ERROR_INVALID_ARGUMENTS);
+  g_clear_error (&error);
+
+
+  /* invalid port */
+  va = g_value_array_new (1);
+  s = gst_structure_new ("aa",
+      "ip", G_TYPE_STRING, "127.0.0.1",
+      "port", G_TYPE_UINT, 0,
+      "username", G_TYPE_STRING, "blah",
+      "password", G_TYPE_STRING, "blah2",
+      NULL);
+  g_value_take_boxed (&val, s);
+  g_value_array_append (va, &val);
+  g_value_take_boxed (&params[0].value, va);
+  st = fs_transmitter_new_stream_transmitter (trans, p, 1, params, &error);
+  ts_fail_unless (st == NULL);
+  ts_fail_unless (error &&
+      error->domain == FS_ERROR &&
+      error->code == FS_ERROR_INVALID_ARGUMENTS);
+  g_clear_error (&error);
+
+  /* no username */
+  va = g_value_array_new (1);
+  s = gst_structure_new ("aa",
+      "ip", G_TYPE_STRING, "127.0.0.1",
+      "port", G_TYPE_UINT, 7654,
+      "password", G_TYPE_STRING, "blah2",
+      NULL);
+  g_value_take_boxed (&val, s);
+  g_value_array_append (va, &val);
+  g_value_take_boxed (&params[0].value, va);
+  st = fs_transmitter_new_stream_transmitter (trans, p, 1, params, &error);
+  ts_fail_unless (st == NULL);
+  ts_fail_unless (error &&
+      error->domain == FS_ERROR &&
+      error->code == FS_ERROR_INVALID_ARGUMENTS);
+  g_clear_error (&error);
+
+  /* no password */
+  va = g_value_array_new (1);
+  s = gst_structure_new ("aa",
+      "ip", G_TYPE_STRING, "127.0.0.1",
+      "port", G_TYPE_UINT, 7654,
+      "username", G_TYPE_STRING, "blah",
       NULL);
   g_value_take_boxed (&val, s);
   g_value_array_append (va, &val);
@@ -714,6 +815,8 @@ GST_START_TEST (test_nicetransmitter_invalid_arguments)
   s = gst_structure_new ("aa",
       "ip", G_TYPE_STRING, "127.0.0.1",
       "port", G_TYPE_UINT, 7654,
+      "username", G_TYPE_STRING, "blah",
+      "password", G_TYPE_STRING, "blah2",
       NULL);
   g_value_take_boxed (&val, s);
   g_value_array_append (va, &val);
@@ -732,6 +835,18 @@ GST_START_TEST (test_nicetransmitter_invalid_arguments)
 }
 GST_END_TEST;
 
+GST_START_TEST (test_nicetransmitter_with_filter)
+{
+  run_nice_transmitter_test (0, NULL, FLAG_RECVONLY_FILTER);
+}
+GST_END_TEST;
+
+GST_START_TEST (test_nicetransmitter_sending_half)
+{
+  run_nice_transmitter_test (0, NULL, FLAG_NOT_SENDING | FLAG_RECVONLY_FILTER);
+}
+GST_END_TEST;
+
 
 static Suite *
 nicetransmitter_suite (void)
@@ -776,6 +891,14 @@ nicetransmitter_suite (void)
   tcase_add_test (tc_chain, test_nicetransmitter_invalid_arguments);
   suite_add_tcase (s, tc_chain);
 
+  tc_chain = tcase_create ("nicetransmitter-with-filter");
+  tcase_add_test (tc_chain, test_nicetransmitter_with_filter);
+  suite_add_tcase (s, tc_chain);
+
+  tc_chain = tcase_create ("nicetransmitter-sending-half");
+  tcase_add_test (tc_chain, test_nicetransmitter_sending_half);
+  suite_add_tcase (s, tc_chain);
+
   return s;
 }
 
