@@ -35,6 +35,9 @@
 #include "generic.h"
 #include "transmitter/rawudp-upnp.h"
 
+#include "stunalternd.h"
+
+
 gint buffer_count[2] = {0, 0};
 GMainLoop *loop = NULL;
 gint candidates[2] = {0, 0};
@@ -48,11 +51,14 @@ gboolean associate_on_source = TRUE;
 gboolean pipeline_done = FALSE;
 GStaticMutex pipeline_mod_mutex = G_STATIC_MUTEX_INIT;
 
+void *stun_alternd_data = NULL;
 
 enum {
   FLAG_HAS_STUN  = 1 << 0,
   FLAG_IS_LOCAL  = 1 << 1,
-  FLAG_NO_SOURCE = 1 << 2
+  FLAG_NO_SOURCE = 1 << 2,
+  FLAG_NOT_SENDING = 1 << 3,
+  FLAG_RECVONLY_FILTER = 1 << 4
 };
 
 #define RTP_PORT 9828
@@ -259,6 +265,15 @@ sync_error_handler (GstBus *bus, GstMessage *message, gpointer blob)
 }
 
 
+static GstElement *
+get_recvonly_filter (FsTransmitter *trans, guint component, gpointer user_data)
+{
+  if (component == 1)
+    return NULL;
+
+  return gst_element_factory_make ("identity", NULL);
+}
+
 static void
 run_rawudp_transmitter_test (gint n_parameters, GParameter *params,
   gint flags)
@@ -275,9 +290,14 @@ run_rawudp_transmitter_test (gint n_parameters, GParameter *params,
   pipeline_done = FALSE;
 
   has_stun = flags & FLAG_HAS_STUN;
-
   associate_on_source = !(flags & FLAG_NO_SOURCE);
 
+  if ((flags & FLAG_NOT_SENDING) && (flags & FLAG_RECVONLY_FILTER))
+  {
+    buffer_count[0] = 20;
+    received_known[0] = 20;
+  }
+
   loop = g_main_loop_new (NULL, FALSE);
   trans = fs_transmitter_new ("rawudp", 2, &error);
 
@@ -288,6 +308,11 @@ run_rawudp_transmitter_test (gint n_parameters, GParameter *params,
 
   ts_fail_if (trans == NULL, "No transmitter create, yet error is still NULL");
 
+  if (flags & FLAG_RECVONLY_FILTER)
+    ts_fail_unless (g_signal_connect (trans, "get-recvonly-filter",
+            G_CALLBACK (get_recvonly_filter), NULL));
+
+
   pipeline = setup_pipeline (trans, G_CALLBACK (_handoff_handler));
 
   bus = gst_element_get_bus (pipeline);
@@ -317,6 +342,8 @@ run_rawudp_transmitter_test (gint n_parameters, GParameter *params,
 
   ts_fail_if (st == NULL, "No stream transmitter created, yet error is NULL");
 
+  g_object_set (st, "sending", !(flags & FLAG_NOT_SENDING), NULL);
+
   ts_fail_unless (g_signal_connect (st, "new-local-candidate",
       G_CALLBACK (_new_local_candidate), GINT_TO_POINTER (flags)),
     "Could not connect new-local-candidate signal");
@@ -350,7 +377,7 @@ run_rawudp_transmitter_test (gint n_parameters, GParameter *params,
 
   g_idle_add (check_running, NULL);
 
-  g_main_run (loop);
+  g_main_loop_run (loop);
 
  skip:
 
@@ -409,13 +436,13 @@ GST_END_TEST;
 
 GST_START_TEST (test_rawudptransmitter_run_invalid_stun)
 {
-  GParameter params[3];
+  GParameter params[4];
 
   /*
    * Hopefully not one is runing a stun server on local port 7777
    */
 
-  memset (params, 0, sizeof (GParameter) * 3);
+  memset (params, 0, sizeof (GParameter) * 4);
 
   params[0].name = "stun-ip";
   g_value_init (&params[0].value, G_TYPE_STRING);
@@ -429,8 +456,11 @@ GST_START_TEST (test_rawudptransmitter_run_invalid_stun)
   g_value_init (&params[2].value, G_TYPE_UINT);
   g_value_set_uint (&params[2].value, 3);
 
-  run_rawudp_transmitter_test (3, params, 0);
+  params[3].name = "upnp-discovery";
+  g_value_init (&params[3].value, G_TYPE_BOOLEAN);
+  g_value_set_boolean (&params[3].value, FALSE);
 
+  run_rawudp_transmitter_test (4, params, 0);
 }
 GST_END_TEST;
 
@@ -608,7 +638,7 @@ GST_START_TEST (test_rawudptransmitter_stop_stream)
 
   g_idle_add (check_running, NULL);
 
-  g_main_run (loop);
+  g_main_loop_run (loop);
 
   gst_element_set_state (pipeline, GST_STATE_NULL);
 
@@ -742,6 +772,159 @@ GST_END_TEST;
 
 #endif /* HAVE_GUPNP */
 
+
+GST_START_TEST (test_rawudptransmitter_with_filter)
+{
+  GParameter params[2];
+
+  memset (params, 0, sizeof (GParameter) * 2);
+
+  params[0].name = "associate-on-source";
+  g_value_init (&params[0].value, G_TYPE_BOOLEAN);
+  g_value_set_boolean (&params[0].value, TRUE);
+
+  params[1].name = "upnp-discovery";
+  g_value_init (&params[1].value, G_TYPE_BOOLEAN);
+  g_value_set_boolean (&params[1].value, FALSE);
+
+  run_rawudp_transmitter_test (2, params,
+      FLAG_RECVONLY_FILTER);
+}
+GST_END_TEST;
+
+GST_START_TEST (test_rawudptransmitter_sending_half)
+{
+  GParameter params[2];
+
+  memset (params, 0, sizeof (GParameter) * 2);
+
+  params[0].name = "associate-on-source";
+  g_value_init (&params[0].value, G_TYPE_BOOLEAN);
+  g_value_set_boolean (&params[0].value, TRUE);
+
+  params[1].name = "upnp-discovery";
+  g_value_init (&params[1].value, G_TYPE_BOOLEAN);
+  g_value_set_boolean (&params[1].value, FALSE);
+
+  run_rawudp_transmitter_test (2, params,
+      FLAG_NOT_SENDING | FLAG_RECVONLY_FILTER);
+}
+GST_END_TEST;
+
+
+GST_START_TEST (test_rawudptransmitter_run_stunalternd)
+{
+  GParameter params[4];
+
+  if (stund_pid <= 0 || stun_alternd_data == NULL)
+    return;
+
+  memset (params, 0, sizeof (GParameter) * 4);
+
+  params[0].name = "stun-ip";
+  g_value_init (&params[0].value, G_TYPE_STRING);
+  g_value_set_static_string (&params[0].value, "127.0.0.1");
+
+  params[1].name = "stun-port";
+  g_value_init (&params[1].value, G_TYPE_UINT);
+  g_value_set_uint (&params[1].value, 3480);
+
+  params[2].name = "stun-timeout";
+  g_value_init (&params[2].value, G_TYPE_UINT);
+  g_value_set_uint (&params[2].value, 5);
+
+  params[3].name = "upnp-discovery";
+  g_value_init (&params[3].value, G_TYPE_BOOLEAN);
+  g_value_set_boolean (&params[3].value, FALSE);
+
+
+  run_rawudp_transmitter_test (3, params, FLAG_HAS_STUN);
+}
+GST_END_TEST;
+
+
+GST_START_TEST (test_rawudptransmitter_run_stun_altern_to_nowhere)
+{
+  GParameter params[4];
+
+  if (stun_alternd_data == NULL)
+    return;
+
+  /*
+   * Hopefully not one is runing a stun server on local port 3478
+   */
+
+  memset (params, 0, sizeof (GParameter) * 4);
+
+  params[0].name = "stun-ip";
+  g_value_init (&params[0].value, G_TYPE_STRING);
+  g_value_set_static_string (&params[0].value, "127.0.0.1");
+
+  params[1].name = "stun-port";
+  g_value_init (&params[1].value, G_TYPE_UINT);
+  g_value_set_uint (&params[1].value, 3480);
+
+  params[2].name = "stun-timeout";
+  g_value_init (&params[2].value, G_TYPE_UINT);
+  g_value_set_uint (&params[2].value, 10);
+
+  params[3].name = "upnp-discovery";
+  g_value_init (&params[3].value, G_TYPE_BOOLEAN);
+  g_value_set_boolean (&params[3].value, FALSE);
+
+  run_rawudp_transmitter_test (4, params, 0);
+}
+GST_END_TEST;
+
+
+void
+setup_stunalternd_valid (void)
+{
+  stun_alternd_data = stun_alternd_init (AF_INET,
+      "127.0.0.1", 3478, 3480);
+
+  if (!stun_alternd_data)
+    g_debug ("Could not spawn stunalternd,"
+        " skipping stun alternate server testing");
+}
+
+static void
+setup_stunalternd_loop (void)
+{
+  stun_alternd_data = stun_alternd_init (AF_INET,
+      "127.0.0.1", 3478, 3478);
+
+  if (!stun_alternd_data)
+    g_debug ("Could not spawn stunalternd,"
+        " skipping stun alternate server testing");
+}
+
+static void
+teardown_stunalternd (void)
+{
+  if (!stun_alternd_data)
+    return;
+
+  stun_alternd_stop (stun_alternd_data);
+  stun_alternd_data = NULL;
+}
+
+static void
+setup_stund_stunalternd (void)
+{
+  setup_stund ();
+  setup_stunalternd_valid ();
+}
+
+
+static void
+teardown_stund_stunalternd (void)
+{
+  teardown_stund ();
+  teardown_stunalternd ();
+}
+
+
 static Suite *
 rawudptransmitter_suite (void)
 {
@@ -799,6 +982,35 @@ rawudptransmitter_suite (void)
   suite_add_tcase (s, tc_chain);
 #endif
 
+  tc_chain = tcase_create ("rawudptransmitter-with-filter");
+  tcase_add_test (tc_chain, test_rawudptransmitter_with_filter);
+  suite_add_tcase (s, tc_chain);
+
+  tc_chain = tcase_create ("rawudptransmitter-sending-half");
+  tcase_add_test (tc_chain, test_rawudptransmitter_sending_half);
+  suite_add_tcase (s, tc_chain);
+
+  tc_chain = tcase_create ("rawudptransmitter-stunalternd");
+  tcase_set_timeout (tc_chain, 5);
+  tcase_add_checked_fixture (tc_chain, setup_stund_stunalternd,
+      teardown_stund_stunalternd);
+  tcase_add_test (tc_chain, test_rawudptransmitter_run_stunalternd);
+  suite_add_tcase (s, tc_chain);
+
+  tc_chain = tcase_create ("rawudptransmitter-stunalternd-to-nowhere");
+  tcase_set_timeout (tc_chain, 12);
+  tcase_add_checked_fixture (tc_chain, setup_stunalternd_valid,
+      teardown_stunalternd);
+  tcase_add_test (tc_chain, test_rawudptransmitter_run_stun_altern_to_nowhere);
+  suite_add_tcase (s, tc_chain);
+
+  tc_chain = tcase_create ("rawudptransmitter-stunalternd-loop");
+  tcase_set_timeout (tc_chain, 12);
+  tcase_add_checked_fixture (tc_chain, setup_stunalternd_loop,
+      teardown_stunalternd);
+  tcase_add_test (tc_chain, test_rawudptransmitter_run_stun_altern_to_nowhere);
+  suite_add_tcase (s, tc_chain);
+
   return s;
 }
 
