@@ -31,7 +31,10 @@
 
 #include "fs-rawudp-marshal.h"
 
-#include "stun.h"
+#include <stun/usages/bind.h>
+#include <stun/stunagent.h>
+#include <stun/usages/timer.h>
+#include <nice/address.h>
 
 #include <gst/farsight/fs-conference-iface.h>
 #include <gst/farsight/fs-interfaces.h>
@@ -63,9 +66,6 @@
 #define DEFAULT_UPNP_MAPPING_TIMEOUT (600)
 #define DEFAULT_UPNP_DISCOVERY_TIMEOUT (10)
 
-#define MAX_STUN_TIMEOUT (60)
-#define DEFAULT_STUN_TIMEOUT (10)
-
 /* Signals */
 enum
 {
@@ -120,7 +120,11 @@ struct _FsRawUdpComponentPrivate
 
   GMutex *mutex;
 
-  gchar stun_cookie[16];
+  StunAgent stun_agent;
+  StunMessage stun_message;
+  guchar stun_buffer[STUN_MAX_MESSAGE_SIZE_IPV6];
+  struct sockaddr_storage stun_sockaddr;
+  gboolean stun_server_changed;
 
   gboolean associate_on_source;
 
@@ -503,6 +507,11 @@ fs_rawudp_component_class_init (FsRawUdpComponentClass *klass)
 
 
   g_type_class_add_private (klass, sizeof (FsRawUdpComponentPrivate));
+
+  if (gst_debug_category_get_threshold (GST_CAT_DEFAULT) >= GST_LEVEL_DEBUG)
+    stun_debug_enable ();
+  else
+    stun_debug_disable ();
 }
 
 
@@ -522,10 +531,8 @@ fs_rawudp_component_init (FsRawUdpComponent *self)
 
   self->priv->associate_on_source = TRUE;
 
-  ((guint32*)self->priv->stun_cookie)[0] = g_random_int ();
-  ((guint32*)self->priv->stun_cookie)[1] = g_random_int ();
-  ((guint32*)self->priv->stun_cookie)[2] = g_random_int ();
-  ((guint32*)self->priv->stun_cookie)[3] = g_random_int ();
+  stun_agent_init (&self->priv->stun_agent,
+      STUN_ALL_KNOWN_ATTRIBUTES, STUN_COMPATIBILITY_RFC3489, 0);
 
 #ifdef HAVE_GUPNP
   self->priv->upnp_mapping = TRUE;
@@ -652,15 +659,21 @@ fs_rawudp_component_stop (FsRawUdpComponent *self)
       self->priv->buffer_recv_id = 0;
     }
 
-    if (self->priv->remote_candidate &&
-        self->priv->sending)
-      fs_rawudp_transmitter_udpport_remove_dest (udpport,
-          self->priv->remote_candidate->ip,
-          self->priv->remote_candidate->port);
-
     if (self->priv->remote_candidate)
+    {
+      if (self->priv->sending)
+        fs_rawudp_transmitter_udpport_remove_dest (udpport,
+            self->priv->remote_candidate->ip,
+            self->priv->remote_candidate->port);
+      else
+        fs_rawudp_transmitter_udpport_remove_recvonly_dest (udpport,
+            self->priv->remote_candidate->ip,
+            self->priv->remote_candidate->port);
+
+
       fs_rawudp_transmitter_udpport_remove_known_address (udpport,
           &self->priv->remote_address, remote_is_unique_cb, self);
+    }
 
     FS_RAWUDP_COMPONENT_UNLOCK (self);
 
@@ -769,11 +782,19 @@ fs_rawudp_component_set_property (GObject *object,
         if (sending != old_sending && candidate)
         {
           if (sending)
+          {
+            fs_rawudp_transmitter_udpport_remove_recvonly_dest (
+                self->priv->udpport, candidate->ip, candidate->port);
             fs_rawudp_transmitter_udpport_add_dest (self->priv->udpport,
                 candidate->ip, candidate->port);
+          }
           else
+          {
             fs_rawudp_transmitter_udpport_remove_dest (self->priv->udpport,
                 candidate->ip, candidate->port);
+            fs_rawudp_transmitter_udpport_add_recvonly_dest (
+                self->priv->udpport, candidate->ip, candidate->port);
+          }
         }
         if (candidate)
           fs_candidate_destroy (candidate);
@@ -986,12 +1007,20 @@ fs_rawudp_component_set_remote_candidate (FsRawUdpComponent *self,
   if (sending)
     fs_rawudp_transmitter_udpport_add_dest (self->priv->udpport,
         candidate->ip, candidate->port);
+  else
+    fs_rawudp_transmitter_udpport_add_recvonly_dest (self->priv->udpport,
+        candidate->ip, candidate->port);
 
   if (old_candidate)
   {
-    fs_rawudp_transmitter_udpport_remove_dest (self->priv->udpport,
-        old_candidate->ip,
-        old_candidate->port);
+    if (sending)
+      fs_rawudp_transmitter_udpport_remove_dest (self->priv->udpport,
+          old_candidate->ip,
+          old_candidate->port);
+    else
+      fs_rawudp_transmitter_udpport_remove_recvonly_dest (self->priv->udpport,
+          candidate->ip,
+          candidate->port);
     fs_candidate_destroy (old_candidate);
   }
 
@@ -1024,11 +1053,11 @@ fs_rawudp_component_maybe_emit_local_candidates (FsRawUdpComponent *self)
   {
     self->priv->local_active_candidate = self->priv->local_upnp_candidate;
     self->priv->local_upnp_candidate = NULL;
-    FS_RAWUDP_COMPONENT_UNLOCK (self);
     GST_DEBUG ("C:%d Emitting UPnP discovered candidate: %s:%u",
         self->priv->component,
-        self->priv->local_upnp_candidate->ip,
-        self->priv->local_upnp_candidate->port);
+        self->priv->local_active_candidate->ip,
+        self->priv->local_active_candidate->port);
+    FS_RAWUDP_COMPONENT_UNLOCK (self);
     fs_rawudp_component_emit_candidate (self,
         self->priv->local_active_candidate);
     return;
@@ -1093,6 +1122,7 @@ _upnp_discovery_timeout (gpointer user_data)
   FsRawUdpComponent *self = user_data;
 
   FS_RAWUDP_COMPONENT_LOCK (self);
+  g_source_unref (self->priv->upnp_discovery_timeout_src);
   self->priv->upnp_discovery_timeout_src = NULL;
   FS_RAWUDP_COMPONENT_UNLOCK (self);
 
@@ -1204,61 +1234,19 @@ fs_rawudp_component_gather_local_candidates (FsRawUdpComponent *self,
 }
 
 static gboolean
-fs_rawudp_component_send_stun (FsRawUdpComponent *self, GError **error)
+fs_rawudp_component_send_stun_locked (FsRawUdpComponent *self, GError **error)
 {
-  struct addrinfo hints;
-  struct addrinfo *result = NULL;
-  struct sockaddr_in address;
-  gchar *packed;
-  guint length;
-  int retval;
-  StunMessage *msg;
-
-  memset (&hints, 0, sizeof (struct addrinfo));
-  hints.ai_family = AF_INET;
-  hints.ai_flags = AI_NUMERICHOST;
-  retval = getaddrinfo (self->priv->stun_ip, NULL, &hints, &result);
-  if (retval != 0)
-  {
-    g_set_error (error, FS_ERROR, FS_ERROR_INVALID_ARGUMENTS,
-        "Invalid IP address %s passed for STUN: %s",
-        self->priv->stun_ip, gai_strerror (retval));
-    return FALSE;
-  }
-  memcpy (&address, result->ai_addr, sizeof (struct sockaddr_in));
-  freeaddrinfo (result);
-
-  address.sin_family = AF_INET;
-  address.sin_port = htons (self->priv->stun_port);
-
-  msg = stun_message_new (STUN_MESSAGE_BINDING_REQUEST,
-      self->priv->stun_cookie, 0);
-  if (!msg)
-  {
-    g_set_error (error, FS_ERROR, FS_ERROR_INTERNAL,
-        "Could not create a new STUN binding request");
-    return FALSE;
-  }
-
-  length = stun_message_pack (msg, &packed);
-
-  if (!fs_rawudp_transmitter_udpport_sendto (self->priv->udpport,
-          packed, length, (const struct sockaddr *)&address, sizeof (address),
-          error))
-  {
-    g_free (packed);
-    stun_message_free (msg);
-    return FALSE;
-  }
-  g_free (packed);
-  stun_message_free (msg);
-
-  return TRUE;
+  return fs_rawudp_transmitter_udpport_sendto (self->priv->udpport,
+      (gchar*) self->priv->stun_buffer,
+      stun_message_length (&self->priv->stun_message),
+      (const struct sockaddr *)&self->priv->stun_sockaddr,
+      sizeof (self->priv->stun_sockaddr), error);
 }
 
 static gboolean
 fs_rawudp_component_start_stun (FsRawUdpComponent *self, GError **error)
 {
+  NiceAddress niceaddr;
   gboolean res = TRUE;
 
   GST_DEBUG ("C:%d starting the STUN process with server %s:%u",
@@ -1270,6 +1258,24 @@ fs_rawudp_component_start_stun (FsRawUdpComponent *self, GError **error)
         self->priv->udpport,
         G_CALLBACK (stun_recv_cb), self);
 
+
+  nice_address_init (&niceaddr);
+  if (!nice_address_set_from_string (&niceaddr, self->priv->stun_ip))
+  {
+    g_set_error (error, FS_ERROR, FS_ERROR_INVALID_ARGUMENTS,
+        "Invalid IP address %s passed for STUN", self->priv->stun_ip);
+    return FALSE;
+  }
+  nice_address_set_port (&niceaddr, self->priv->stun_port);
+  nice_address_copy_to_sockaddr (&niceaddr,
+      (struct sockaddr *) &self->priv->stun_sockaddr);
+
+  stun_usage_bind_create (
+      &self->priv->stun_agent,
+      &self->priv->stun_message,
+      self->priv->stun_buffer,
+      sizeof(self->priv->stun_buffer));
+
   if (self->priv->stun_timeout_thread == NULL) {
     /* only create a new thread if the old one was stopped. Otherwise we can
      * just reuse the currently running one. */
@@ -1314,8 +1320,15 @@ stun_recv_cb (GstPad *pad, GstBuffer *buffer,
 {
   FsRawUdpComponent *self = FS_RAWUDP_COMPONENT (user_data);
   FsCandidate *candidate = NULL;
-  StunMessage *msg;
-  StunAttribute **attr;
+  StunMessage msg;
+  StunValidationStatus stunv;
+  StunUsageBindReturn stunr;
+  struct sockaddr_storage addr;
+  socklen_t addr_len = sizeof(addr);
+  struct sockaddr_storage alt_addr;
+  socklen_t alt_addr_len = sizeof(alt_addr);
+  gchar addr_str[NI_MAXHOST];
+  NiceAddress niceaddr;
 
   if (GST_BUFFER_SIZE (buffer) < 4)
     /* Packet is too small to be STUN */
@@ -1328,69 +1341,69 @@ stun_recv_cb (GstPad *pad, GstBuffer *buffer,
 
   g_assert (fs_rawudp_transmitter_udpport_is_pad (self->priv->udpport, pad));
 
-  msg = stun_message_unpack (GST_BUFFER_SIZE (buffer),
-      (const gchar *) GST_BUFFER_DATA (buffer));
-  if (!msg)
-    /* invalid message */
+  FS_RAWUDP_COMPONENT_LOCK(self);
+  stunv = stun_agent_validate (&self->priv->stun_agent, &msg,
+      GST_BUFFER_DATA (buffer), GST_BUFFER_SIZE (buffer), NULL, NULL);
+  FS_RAWUDP_COMPONENT_UNLOCK(self);
+
+  /* not a valid stun message */
+  if (stunv != STUN_VALIDATION_SUCCESS)
     return TRUE;
 
-  if (memcmp (msg->transaction_id, self->priv->stun_cookie, 16) != 0)
+  stunr = stun_usage_bind_process (&msg,
+      (struct sockaddr *) &addr, &addr_len,
+      (struct sockaddr *) &alt_addr, &alt_addr_len);
+
+  switch (stunr)
   {
-    /* not ours */
-    stun_message_free (msg);
-    return TRUE;
-  }
-
-  if (msg->type == STUN_MESSAGE_BINDING_ERROR_RESPONSE)
-  {
-    fs_rawudp_component_emit_error (FS_RAWUDP_COMPONENT (self),
-        FS_ERROR_NETWORK, "Got an error message from the STUN server",
-        "The STUN process produced an error");
-    stun_message_free (msg);
-    // fs_rawudp_component_stop_stun (self, component_id);
-    /* Lets not stop the STUN now and wait for the timeout
-     * in case the server answers with the right reply
-     */
-    return FALSE;
-  }
-
-  if (msg->type != STUN_MESSAGE_BINDING_RESPONSE)
-  {
-    stun_message_free (msg);
-    return TRUE;
-  }
-
-
-  for (attr = msg->attributes; *attr; attr++)
-  {
-    if ((*attr)->type == STUN_ATTRIBUTE_MAPPED_ADDRESS)
-    {
-      // TODO
-      gchar *id = g_strdup_printf ("L1");
-      gchar *ip = g_strdup_printf ("%u.%u.%u.%u",
-          ((*attr)->address.ip & 0xff000000) >> 24,
-          ((*attr)->address.ip & 0x00ff0000) >> 16,
-          ((*attr)->address.ip & 0x0000ff00) >>  8,
-          ((*attr)->address.ip & 0x000000ff));
-
-      candidate = fs_candidate_new (id,
-          self->priv->component,
-          FS_CANDIDATE_TYPE_SRFLX,
-          FS_NETWORK_PROTOCOL_UDP,
-          ip,
-          (*attr)->address.port);
-      g_free (id);
-      g_free (ip);
-
-      GST_DEBUG ("Stun server says we are %u.%u.%u.%u %u\n",
-          ((*attr)->address.ip & 0xff000000) >> 24,
-          ((*attr)->address.ip & 0x00ff0000) >> 16,
-          ((*attr)->address.ip & 0x0000ff00) >>  8,
-          ((*attr)->address.ip & 0x000000ff),(*attr)->address.port);
+    case STUN_USAGE_BIND_RETURN_INVALID:
+      /* Not a valid bind reponse */
+      return TRUE;
+    case STUN_USAGE_BIND_RETURN_ERROR:
+      /* Not a valid bind reponse */
+      return FALSE;
+    case STUN_USAGE_BIND_RETURN_ALTERNATE_SERVER:
+      /* Change servers and reset timeouts */
+      FS_RAWUDP_COMPONENT_LOCK(self);
+      memcpy (&self->priv->stun_sockaddr, &alt_addr,
+          MIN (sizeof(self->priv->stun_sockaddr), alt_addr_len));
+      self->priv->stun_server_changed = TRUE;
+      stun_usage_bind_create (
+          &self->priv->stun_agent,
+          &self->priv->stun_message,
+          self->priv->stun_buffer,
+          sizeof(self->priv->stun_buffer));
+      nice_address_init (&niceaddr);
+      nice_address_set_from_sockaddr (&niceaddr,
+          (const struct sockaddr *) &alt_addr);
+      nice_address_to_string (&niceaddr, addr_str);
+      GST_DEBUG ("Stun server redirected us to alternate server %s:%d",
+          addr_str, nice_address_get_port (&niceaddr));
+      if (self->priv->stun_timeout_id)
+        gst_clock_id_unschedule (self->priv->stun_timeout_id);
+      FS_RAWUDP_COMPONENT_UNLOCK(self);
+      return FALSE;
+    default:
+      /* For any other case, pass the packet through */
+      return TRUE;
+    case STUN_USAGE_BIND_RETURN_SUCCESS:
       break;
-    }
   }
 
+  nice_address_init (&niceaddr);
+  nice_address_set_from_sockaddr (&niceaddr, (const struct sockaddr *) &addr);
+  nice_address_to_string (&niceaddr, addr_str);
+
+  candidate = fs_candidate_new ("L1",
+      self->priv->component,
+      FS_CANDIDATE_TYPE_SRFLX,
+      FS_NETWORK_PROTOCOL_UDP,
+      addr_str,
+      nice_address_get_port (&niceaddr));
+
+  GST_DEBUG ("Stun server says we are %s:%u\n", addr_str,
+      nice_address_get_port (&niceaddr));
+
   FS_RAWUDP_COMPONENT_LOCK(self);
   fs_rawudp_component_stop_stun_locked (self);
 #ifdef HAVE_GUPNP
@@ -1408,8 +1421,6 @@ stun_recv_cb (GstPad *pad, GstBuffer *buffer,
 
   fs_candidate_destroy (candidate);
 
-  /* It was a stun packet, lets drop it */
-  stun_message_free (msg);
   return FALSE;
 }
 
@@ -1422,8 +1433,11 @@ stun_timeout_func (gpointer user_data)
   gboolean emit = TRUE;
   GstClockTime next_stun_timeout;
   GError *error = NULL;
-  guint next_timeout_ms = 100;
   guint timeout_accum_ms = 0;
+  guint remainder;
+  StunUsageTimerReturn timer_ret = STUN_USAGE_TIMER_RETURN_RETRANSMIT;
+  StunTransactionId stunid;
+  StunTimer stun_timer;
 
   sysclock = gst_system_clock_obtain ();
   if (sysclock == NULL)
@@ -1435,12 +1449,22 @@ stun_timeout_func (gpointer user_data)
   }
 
   FS_RAWUDP_COMPONENT_LOCK(self);
+  stun_timer_start (&stun_timer);
+
   while (!self->priv->stun_stop &&
       timeout_accum_ms < self->priv->stun_timeout * 1000)
   {
-    FS_RAWUDP_COMPONENT_UNLOCK(self);
-    if (!fs_rawudp_component_send_stun (self, &error))
+    if (self->priv->stun_server_changed)
     {
+      stun_timer_start (&stun_timer);
+      self->priv->stun_server_changed = FALSE;
+      timer_ret = STUN_USAGE_TIMER_RETURN_RETRANSMIT;
+    }
+
+    if (timer_ret == STUN_USAGE_TIMER_RETURN_RETRANSMIT &&
+        !fs_rawudp_component_send_stun_locked (self, &error))
+    {
+      FS_RAWUDP_COMPONENT_UNLOCK(self);
       fs_rawudp_component_emit_error (self, error->code, "Could not send stun",
           error->message);
       g_clear_error (&error);
@@ -1448,19 +1472,20 @@ stun_timeout_func (gpointer user_data)
       fs_rawudp_component_stop_stun_locked (self);
       goto interrupt;
     }
-    FS_RAWUDP_COMPONENT_LOCK(self);
 
     if (self->priv->stun_stop)
       goto interrupt;
 
+    remainder = stun_timer_remainder (&stun_timer);
+
     next_stun_timeout = gst_clock_get_time (sysclock) +
-      next_timeout_ms * GST_MSECOND;
+      remainder * GST_MSECOND;
 
     id = self->priv->stun_timeout_id = gst_clock_new_single_shot_id (sysclock,
         next_stun_timeout);
 
     GST_LOG ("C:%u Waiting for STUN reply for %u ms, next: %u ms",
-        self->priv->component, next_timeout_ms, timeout_accum_ms);
+        self->priv->component, remainder, timeout_accum_ms);
 
     FS_RAWUDP_COMPONENT_UNLOCK(self);
     gst_clock_id_wait (id, NULL);
@@ -1469,9 +1494,11 @@ stun_timeout_func (gpointer user_data)
     gst_clock_id_unref (id);
     self->priv->stun_timeout_id = NULL;
 
-    next_timeout_ms *= 2;
-    next_timeout_ms += 100;
-    timeout_accum_ms += next_timeout_ms;
+    timer_ret = stun_timer_refresh (&stun_timer);
+    timeout_accum_ms += remainder;
+
+    if (timer_ret == STUN_USAGE_TIMER_RETURN_TIMEOUT)
+      break;
   }
 
  interrupt:
@@ -1483,6 +1510,9 @@ stun_timeout_func (gpointer user_data)
 
   fs_rawudp_component_stop_stun_locked (self);
 
+  stun_message_id (&self->priv->stun_message, stunid);
+  stun_agent_forget_transaction (&self->priv->stun_agent, stunid);
+
   FS_RAWUDP_COMPONENT_UNLOCK(self);
 
   gst_object_unref (sysclock);
